@@ -185,13 +185,14 @@ class Collection(BaseModel):
     def create_index(self,
                      tag='latest',
                      target_library=IndexLibraries.hnswlib,
-                     metric=DistanceMetric.cosine,                     
+                     metric=DistanceMetric.cosine,
+                     target_recall=None,
                      M=None,
                      ef=None):
-
         body = {'tag': tag,
                 'target_library': target_library,
                 'metric': metric,
+                'target_recall': target_recall,
                 'hnswlib_M':  M,
                 'hnswlib_ef': ef}
         resp = session.post("/collections/%d/index" % self.id, json=body)
@@ -230,7 +231,7 @@ class Collection(BaseModel):
             self._update()
         return object.__getattribute__(self, attr)        
 
-
+        
 def create_collection(name:str):
     """
     Create a collection of the specified name
@@ -268,6 +269,8 @@ class Index(BaseModel):
     metric:        str = Field(description='The distance metric ("space" in hnswlib): "cosine", "ip", or "l2"')
     hnswlib_M:  Optional[int] = Field(default=None, description="The M value passed to hnswlib when creating the index.")
     hnswlib_ef: Optional[int]  = Field(default=None, description="The ef_construction value passed to hnswlib when creating the index.")
+    hnswlib_ef_search: Optional[int] = Field(default=None, ge=10, description="The recommended ef value to use at search time.")
+    target_recall: Optional[float] = Field(default=None, ge=0.5, le=1, description="The desired recall value to target for index parameter optimization.")        
     count: int = Field(description="The number of vectors included in the index.  The number of vectors in the collection at the time of index build.")
     created_at: float = Field(description='The epoch timestamp when the index was requested to be created.')
     state: IndexBuildState = Field(description = "The current build status.")
@@ -278,12 +281,12 @@ class Index(BaseModel):
     
     def _wait_complete(self):
         while self.state not in [IndexBuildState.failed, IndexBuildState.complete]:
-            print("Waiting for index completion...")  # XXX provide percentage completion based on completed_at estimate
+            print(f"Index Status: {self.build_status}")  # XXX provide percentage completion
             data = {'tag': self.tag}
             resp = session.get (f"/collections/{self.collection.id}/index?{urllib.parse.urlencode(data)}")
             items = resp.json()['items'][0]
             self.__dict__.update(items)
-            sleep(5)  # XXX update sleep time based on something like halfway toward the estimated remaining completed_at estimate
+            sleep(2)  # XXX update sleep time based on something like halfway toward the estimated remaining completed_at estimate
         if self.state == IndexBuildState.failed:
             raise IndexError(self.build_status)
         
@@ -300,6 +303,8 @@ class Index(BaseModel):
         open(fname, 'wb').write(resp.content)
         ix = hnswlib.Index(space=self.metric, dim=self.collection.dimension)
         ix.load_index(fname, max_elements=max_elements)
+        ef = self.hnswlib_ef_search if self.hnswlib_ef_search else self.hnswlib_ef
+        ix.set_ef(ef)
         return ix
 
 
